//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TParticle: defines  equivalent of HEPEVT particle                    //
// (cloned from TPrimary)                                               //
// Jan 08 1998 P.Murat (CDF)                                            //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TParticle
#define ROOT_TParticle

#ifndef ROOT_TNamed
//*KEEP,TNamed.
#include "TNamed.h"
//*KEND.
#endif
#ifndef ROOT_TAttLine
//*KEEP,TAttLine.
#include "TAttLine.h"
//*KEND.
#endif
#ifndef ROOT_X3DBuffer
//*KEEP,X3DBuffer,T=C.
#include "X3DBuffer.h"
//*KEND.
#endif
#ifndef ROOT_TLorentzVector
//*KEEP,TLORENTZVECTOR.
#include "TLorentzVector.h"
//*KEND.
#endif
#ifndef ROOT_TParticlePDG
//*KEEP,TParticlePDG,T=C++.
#include "TParticlePDG.h"
//*KEND.
#endif

class TParticle : public TObject, public TAttLine  {
protected:

  Int_t          fPdgCode;		// PDG code of the particle
  Int_t          fStatusCode;		// generation status code
  Int_t          fMother[2];		// Indices of the mother particles
  Int_t          fDaughter[2];		// Indices of the daughter particles
  Float_t        fWeight;               // particle weight

  TLorentzVector fMomentum;	        // 4-momentum of the particle
  TLorentzVector fProductionVertex;     // Production vertex

  TParticlePDG*  fParticlePDG;          //! reference to the particle record in PDG database
//------------------------------------------------------------------------------
//  functions
//------------------------------------------------------------------------------
 public:
				// ****** constructors and destructor
  TParticle();

  TParticle(Int_t pdg, Int_t status,
	    Int_t mother1, Int_t mother2,
	    Int_t daughter1, Int_t daughter2,
	    Double_t px, Double_t py, Double_t pz, Double_t etot,
	    Double_t vx, Double_t vy, Double_t vz, Double_t time);

  virtual ~TParticle();


//   virtual TString* Name   () const { return fName.Data(); }
//   virtual Text_t*  GetName() const { return fName.Data(); }

  Int_t          GetStatusCode   () { return fStatusCode; }
  Int_t          GetPdgCode      () { return fPdgCode; }
  Int_t          GetFirstMother  () { return fMother[0]; }
  Int_t          GetSecondMother () { return fMother[1]; }
  Int_t          GetFirstDaughter() { return fDaughter[0]; }
  Int_t          GetLastDaughter () { return fDaughter[1]; }
  Int_t          GetNDaughters   () { return fDaughter[1]-fDaughter[0]+1; }
  Float_t        GetWeight       () {return fWeight;}
  const TParticlePDG*  GetPDG    () const { return fParticlePDG; }

  Int_t          Beauty          () { return fParticlePDG->Beauty(); }
  Int_t          Charm           () { return fParticlePDG->Charm(); }
  Int_t          Strangeness     () { return fParticlePDG->Strangeness(); }

  const TLorentzVector* Momentum() const { return &fMomentum; }
  const TLorentzVector* ProductionVertex() const { return &fProductionVertex; }

				// ****** redefine several most oftenly used
				// methods of LORENTZ_VECTOR

  Double_t         Vx    () { return fProductionVertex.X(); }
  Double_t         Vy    () { return fProductionVertex.Y(); }
  Double_t         Vz    () { return fProductionVertex.Z(); }
  Double_t         T     () { return fProductionVertex.T(); }

  Double_t         Px    () { return fMomentum.Px(); }
  Double_t         Py    () { return fMomentum.Py(); }
  Double_t         Pz    () { return fMomentum.Pz(); }
  Double_t         P     () const { return fMomentum.P(); }
  Double_t         Pt    () { return fMomentum.Pt(); }
  Double_t         Energy() { return fMomentum.Energy(); }
  Double_t         Eta   () { return fMomentum.Eta(); }
  Double_t         Phi   () { return fMomentum.Phi(); }

				// setters

  void           SetMother  (int i, int code)  { fMother[i]   = code ; }
  void           SetDaughter(int i, int code)  { fDaughter[i] = code ; }
  void           SetWeight(Float_t weight = 1) { fWeight = weight; }

				// ****** overloaded functions of TObject

  virtual Bool_t    Is3D() {return kTRUE;}
  virtual void      Paint(Option_t *option = "");
  virtual void      Print(Option_t *option = "");
  virtual void      Sizeof3D() const;
  virtual Int_t     DistancetoPrimitive(Int_t px, Int_t py);
  virtual void      ExecuteEvent(Int_t event, Int_t px, Int_t py);
  virtual const     Text_t *GetName() const;
  virtual const     Text_t *GetTitle() const;


  ClassDef(TParticle,1)  // TParticle vertex particle information
};

#endif

